<?php
 
/**
 * Plugin Name:       LWS SMS
 * Plugin URI:        https://www.lws.fr/
 * Description:       With LWS SMS, create SMS Templates and configurate your website to send SMS to clients when you want it !
 * Version:           1.2.5
 * Author:            LWS
 * Author URI:        https://www.lws.fr
 * Tested up to:      5.9.3
 * Domain Path:       /languages
 * 
 * @link              https://sms.lws.fr/
 * @since             1.0
 * @package           lws-sms
*/

if ( ! defined( 'ABSPATH' ) ) {
    exit; //Exit if accessed directly
}
register_activation_hook( __FILE__, 'lws_sms_cron_lws' );
function lws_sms_cron_lws() {
    if (!wp_next_scheduled ( 'check_sender_cron')){
        wp_schedule_event( time(), 'hourly', 'check_sender_cron' );
    }
}

add_action('init', 'lws_sms_define_hooks');
function lws_sms_define_hooks(){
    define('HOOKS', array('completed', 'processing', 'cancelled', 'stock', 'coupon', 'sessions'));
    define('HOOKS_TRAD', array(__('Order completed', 'lws-sms'), __('Order processing', 'lws-sms'),  __('Order cancelled', 'lws-sms'), __('Product in stock', 'lws-sms'), __('New coupon', 'lws-sms'), __("Cart abandonned", "lws-sms")));
    define('ADMIN_HOOKS', array('admin_new_order', 'admin_dailies'));
    define('ADMIN_HOOKS_TRAD', array(__('New order', 'lws-sms'), __('Daily Logs (Previous day)', 'lws-sms')));
    define('HOOKS_CLIENTS_TRAD', array(__('Order shipped', 'lws-sms'), __('Payment accepted', 'lws-sms')));
}

//Add cron to check senders & update the list
add_action( 'check_sender_cron', 'lws_sms_check_sender' );
function lws_sms_check_sender() {
    $response = lws_sms_APILoadSender();
    $get_data = json_decode($response, true);
    if($get_data['code'] === '100'){
        foreach($get_data['list_senderid'] as $sender){
            if ($sender['status'] == 'unblock'){
                $sender_list[$sender['id']] = $sender['sender_id'];
            }
        }
        update_option("lws_sender_id", $sender_list);
    }
}

//Add cron to check balance & send mail if needed
add_action( 'check_balance_cron', 'lws_sms_check_balance', 10, 2 );
function lws_sms_check_balance($user, $limit){
    if ($limit >= $SMSBalance = json_decode(lws_sms_APIBalance(), true)['solde']){
        
        $html  = (get_locale() == 'fr_FR') ? file_get_contents(__DIR__ . '/mail/mail-alerte-sms-fr.html') : file_get_contents(__DIR__ . '/mail/mail-alerte-sms.html');
        $html = chunk_split ( base64_encode ( $html ), 70 );

        // To send HTML mail, the Content-type header must be set
        $headers[] = 'MIME-Version: 1.0';
        $headers[] = 'Content-Type: text/html; charset="utf-8"';
        $headers[] = 'Content-Transfer-Encoding: base64';        
        // En-têtes additionnels
        $name = get_bloginfo('name','');
        $site = sanitize_text_field($_SERVER['SERVER_NAME']);
        $headers[] = 'FROM: '. esc_html($name) .' <noreply@'. esc_html($site) . '>';
        $headers[] = 'In-Reply-To: <e08d4521220acc90df005ae3231c8fef20549b62@'.esc_html($site).'>';

        $result = wp_mail( esc_html($user->user_email), esc_html__("Low SMS Balance Alert", "lws-sms"), $html, implode("\r\n", $headers));
    }
    
}

add_action('daily_logs_cron', 'lws_sms_daily_cron');
function lws_sms_daily_cron(){
    //Get every users, sessions and orders
    global $wpdb;
    $order_created = array();
    $users = array();
    $sessions = array();
    $sales = 0;
    $product = 0;
    $turnover = 0;
    $usage = "dailies";
    $users_registered = $wpdb->get_results("SELECT * FROM ". $wpdb->prefix."users");
    $order_stats = $wpdb->get_results("SELECT * FROM ". $wpdb->prefix."wc_order_stats");
    $order_sessions = $wpdb->get_results("SELECT * FROM ". $wpdb->prefix."woocommerce_sessions");
    $product_lookup = $wpdb->get_results("SELECT * FROM ". $wpdb->prefix."wc_product_meta_lookup");
    
    //Check for every lost carts of clients
    foreach ($order_sessions as $session){
        //Check expiry of the session
        $expire = ($session->session_expiry);
        //Get their cart if any
        $cart = unserialize(unserialize($session->session_value)['cart']);
        
        //If it is a lost cart and said cart is not empty, count it
        if ($expire - time() <= 43200 && $expire - time() > 0 && !empty($cart)){
            $sessions[] = $session;
        }
    }

    //Check for the best sellings products
    foreach ($product_lookup as $product){
        //Check for the total sales
        if ($product->total_sales > $sales){
            $sales = $product->total_sales;
            $product_best = wc_get_product($product->product_id);
        }
    }
    $product = $product_best->name . __(" | Sales: ", "lws-sms") . $sales;
    
    //Check for every user created the day before the cron's activation
    foreach ($users_registered as $user){
        if (explode(" ", $user->user_registered)[0] == (gmdate("Y-m-d", time() - 86400))){
            $users[] = $user;
        }
    }
    
    //Check for every orders created the day before the cron activation
    foreach ($order_stats as $stats){
        if (explode(" ", $stats->date_created_gmt)[0] == (gmdate("Y-m-d", time() - 86400))){
            $order_created[] = $stats;
            //If the paiement is accepted
            if ($stats->status == "wc-processing"){
                //Total amount gained that day
                $turnover += floatval($stats->net_total);
            }
        }
    }
    
    //Stock the amount of new orders
    $new_orders = count($order_created);
    
    //Stock the amount of new users
    $new_users = count($users);
    
    //Stock the amount of lost carts
    $lost_carts = count($sessions);
    
    //Send SMS
    $admin_check = get_option('lws_checked_options');
    //If activated
    if ($admin_check[$usage]['is_checked']){
        //Only if client has authorized SMS for this
        $checkboxes = array();
        get_user_meta( $user->ID, 'checkboxes', true) ? 
        $checkboxes = get_user_meta( $user->ID, 'checkboxes', true) : update_user_meta($user->ID, 'checkboxes', $checkboxes);
        
        if (count($checkboxes) == 0 || $checkboxes[$usage]['is_checked']){
            $models = get_option('lws_model_list');
            //Get every user who can manage woocommerce (admin)
            $users = get_users();
            $users_can = array();
            foreach ( $users as $user ) {
                if (user_can($user, 'manage_woocommerce')){
                    $users_can[] = $user;
                }
            }
            //Get their phone numbers
            $num = "";
            $count = 0;
            foreach ($users_can as $user){
                if ($user->phone_sms){
                    $num = $num . $user->phone_sms . ",";
                    $count++;
                }
            }
            foreach ($models as $model){
                foreach($model['in_use'] as $in_use){
                    if ($in_use == $usage){
                        if ((json_decode(lws_sms_APIBalance(), true)['solde'] - $model['nb_sms']) - $count >= 0){                        
                            $message = $model['message'];
                            $message = str_replace("[[PanierAbandonnes]]",  $lost_carts, $message);
                            $message = str_replace("[[MontantTotal]]",  $turnover, $message);
                            $message = str_replace("[[Commandes]]",  $new_orders, $message);
                            $message = str_replace("[[NouveauxClients]]",  $new_users, $message);
                            $message = str_replace("[[MeilleureVente]]",  $product, $message);
                            $message = str_replace("[[NomSite]]", get_bloginfo('name'), $message);
                            $message = str_replace("[[URLSite]]", get_bloginfo('url'), $message);
                            $message = str_replace("[[Date]]",gmdate("[Y-m-d] - H:i:s : ", time()) , $message);
                            lws_sms_APISendSMS($model['sender'], $num, $message);
                            break;
                        }
                    }
                }
            }
        }
    }
}

//On deactivation, delete every crons
register_deactivation_hook( __FILE__, 'lws_sms_cron_lws_off' );
function lws_sms_cron_lws_off() {
    wp_clear_scheduled_hook( 'check_sender_cron' );
    $args = array(wp_get_current_user(), get_option('lws_sms_alert'));
    wp_clear_scheduled_hook( 'check_balance_cron', $args);
    wp_clear_scheduled_hook( 'check_stock_alert_cron' );
    wp_clear_scheduled_hook( 'check_sessions_cron');
    wp_clear_scheduled_hook( 'daily_logs_cron');
}

//When uninstalled, delete all data
register_uninstall_hook(__FILE__, "lws_sms_delete_LWS_options");
function lws_sms_delete_LWS_options(){
    delete_option("lws_sender_id");
    delete_option("lws_user_sms");
    delete_option("lws_model_list");
    delete_option("lws_checked_options");
    delete_option("lws_checked_client");
    delete_user_meta(wp_get_current_user(), 'phone_sms');
    delete_user_meta(wp_get_current_user(), 'phone_sms_print');
}

//Load CSS in wp-admin
add_action('admin_enqueue_scripts', 'lws_sms_setting_up_scripts');
function lws_sms_setting_up_scripts() {
    wp_enqueue_style( 'config_css', plugins_url('css/config_page.css', __FILE__));
    wp_enqueue_style( 'config_css_specialized', plugins_url('css/config_page_specialized.css', __FILE__));
    wp_enqueue_style( 'dt_css', plugins_url('css/jquery.dataTables.min.css', __FILE__));
    wp_enqueue_style( 'dt_resp_css', plugins_url('css/responsive.dataTables.min.css', __FILE__));
    wp_enqueue_script( 'dt_js', plugins_url('js/jquery.dataTables.min.js', __FILE__));
    wp_enqueue_script( 'dt_resp_js', plugins_url('js/dataTables.responsive.min.js', __FILE__));

}

//Load translations
add_action('init', 'lws_sms_add_textdomain');
function lws_sms_add_textdomain(){ load_plugin_textdomain('lws-sms', false, dirname( plugin_basename( __FILE__ ) ) . '/languages' ); }

//Create menu and submenus in the dashboard (wp-admin)
add_action('admin_menu', 'lws_sms_add_menu');
function lws_sms_add_menu(){
    $menu_slug = 'sms-api-settings';
    add_menu_page( __('LWS SMS Settings', 'lws-sms'), 'LWS SMS', 'read', $menu_slug, 'lws_sms_config', plugins_url('/images/plugin_lws_sms.svg', __FILE__), 58 );
}


//Everything related to the general options page
function lws_sms_config(){
    //Check if form is filled to connect to the API
    if (isset($_POST['validate_connexion'])){
        $username = sanitize_text_field($_POST['username']);
        $apikey = sanitize_text_field($_POST['APIKey']);
        if (!empty($username) && !empty($apikey)) {
            //API Call to get infos on user
            $get_data = json_decode(lws_sms_APIInfo($username, $apikey)['body'], true);
        }

        //If OK, $sender get every senders, we get infos on the user and save everything
        if($get_data['code'] === '100'){
            foreach($get_data['list_senderid'] as $s){
                $sender[$s['id']] = $s['sender_id'];
            }

            $user_sms = (array('username' => $get_data['username'], 'api_key' => $get_data['api_key_client'], 'id_client' => $get_data['id_client']));
            
            update_option("lws_sender_id", $sender);
            update_option("lws_user_sms", $user_sms);
            if (!get_option('lws_sms_alert')){ update_option("lws_sms_alert", 1); }
        }
        //Error management
        else {
            $formError = true;
            switch ($get_data['code']) {
                case '101':
                    $formErrorMessage = __("Invalid username or API Key", "lws-sms");
                    break;
                case '102':
                    $formErrorMessage = __("Please fill in your username", "lws-sms");
                    break;
                case '103':
                    $formErrorMessage = __("Please fill in your API Key", "lws-sms");
                    break;
                //When the error is unknown
                default:
                    $formErrorMessage = __("Something went wrong with our API", "lws-sms");
                    break;
            }
        }
    }

    $active_tab = isset( $_GET[ 'tab' ] ) ? sanitize_text_field($_GET[ 'tab' ]) : 'welcome';
        //If user is in Welcome
        //
        if ($active_tab == 'welcome'){
            
            //On disconnect, delete everything
            if (isset($_POST['disconnect_user'])) {
                delete_option("lws_sender_id");
                delete_option("lws_user_sms");
                delete_option("lws_sms_alert");
            }
            //Launch the basic layout for the settings
            include __DIR__ . '/view/config_base.php';
            if (get_option("lws_user_sms") && get_option("lws_sender_id")){
                if (lws_sms_APICheckAPI() == '101'){
                    include __DIR__ . "/view/alert/apikey_wrong.php";
                }
            }
            include __DIR__ . '/view/welcome.php';
        }
        //END Welcome
        //
        
        else{
            //Launch the basic layout for the settings
            include __DIR__ . '/view/config_base.php';
            if (get_option("lws_user_sms") && get_option("lws_sender_id")){
                if (lws_sms_APICheckAPI() == '101'){
                    include __DIR__ . "/view/alert/apikey_wrong.php";
                }
            }
        }
    
    //If connected
    if (get_option("lws_user_sms") && get_option("lws_sender_id")){
        //If user is in General//
        //
        if ($active_tab == 'general'){
            if (isset($_POST['change_alert_sms'])) { 
                $args_delete = array(wp_get_current_user(), get_option('lws_sms_alert'));
                wp_clear_scheduled_hook( 'check_balance_cron', $args_delete);
                update_option("lws_sms_alert", sanitize_text_field($_POST['alert_sms']));
                $args = array(wp_get_current_user(), sanitize_text_field($_POST['alert_sms']));
                if (!wp_next_scheduled ( 'check_balance_cron', $args_delete ) && !wp_next_scheduled ( 'check_balance_cron', $args) ) {
                    wp_schedule_event( time(), 'daily', 'check_balance_cron', $args);
                }
            }

            //Balance, alert and senders stocked for later use in the page
            $SMSBalance = json_decode(lws_sms_APIBalance(), true)['solde'];
            $sender_ids = get_option("lws_sender_id");
            $list_history = json_decode(lws_sms_APISMSHistory(), true)['sms_history'];
            $alert_sms = get_option('lws_sms_alert');

            include __DIR__ . '/view/config_page.php';
        }
        //END General//
        //

        //If user is in model//
        //
        if ($active_tab == 'templates'){ 
            $model_error = false;
            $edit_model = false;
            //Adding a new model to the list
            if (isset($_POST['validate_model'])) {
                if (!empty(sanitize_text_field($_POST['model_name'])) && !empty(sanitize_text_field($_POST['senders_select'])) && !empty(sanitize_textarea_field($_POST['sms_model']))) {
                    $model_error = false;
                    $already_done = false;
                    if (!$models = get_option('lws_model_list')){
                        $models[] = array('id' => uniqid('', true), 'name' => sanitize_text_field($_POST['model_name']), 'sender' => sanitize_text_field($_POST['senders_select']), 'message' => sanitize_textarea_field($_POST['sms_model']), 'nb_sms' => sanitize_text_field($_POST['number_message']), 'in_use' => array('NO'));
                        update_option('lws_model_list', $models);
                        $already_done = true;
                    }
                    else{
                        //Check if name is already used
                        foreach ($models as $key => $value){ 
                            if ($models[$key]['name'] == sanitize_text_field($_POST['model_name']) && !isset($_POST['editing'])){
                                $model_error = true;
                                $model_error_message = __("This model already exist, choose another name", "lws-sms");
                            }
                            else if ($models[$key]['name'] == sanitize_text_field($_POST['model_name']) && isset($_POST['editing']) && $models[$key]['id'] != sanitize_text_field($_POST['model_id'])){
                                $model_error = true;
                                $model_error_message = __("This model already exist, choose another name", "lws-sms");
                            }
                        }
                    }

                    if (!$model_error && !isset($_POST['editing']) && !$already_done){
                        $models[] = array('id' => uniqid('', true), 'name' => sanitize_text_field($_POST['model_name']), 'sender' => sanitize_text_field($_POST['senders_select']), 'message' => sanitize_textarea_field($_POST['sms_model']), 'nb_sms' => sanitize_textarea_field($_POST['number_message']), 'in_use' => array('NO'));
                        update_option('lws_model_list', $models);
                    }
                    else if (!$model_error && isset($_POST['editing']) && !$already_done){
                        $model_key = sanitize_text_field($_POST['model_key']);
                        $models[$model_key] = array('id' => $models[$model_key]['id'], 'name' => sanitize_text_field($_POST['model_name']), 'sender' => sanitize_text_field($_POST['senders_select']), 'message' => sanitize_textarea_field($_POST['sms_model']), 'nb_sms' => sanitize_textarea_field($_POST['number_message']), 'in_use' => $models[$model_key]['in_use']);
                        update_option('lws_model_list', $models);                            
                    }
                }
            }
            
            $models = get_option('lws_model_list');

            if (isset($_POST['edit_model'])) {
                foreach ($models as $key => $model){
                    if ($model['name'] == sanitize_text_field($_POST['model_name'])){ $edit_model = $model; $edit_key = $key; $edit_id = $model['id'];}
                }
            }

            //Delete the selected model 
            if (isset($_POST['delete_model'])) {
                foreach ($models as $key => $value){
                    if ($models[$key]['name'] == sanitize_text_field($_POST['model_name'])){ unset($models[$key]); break; }
                }
                update_option('lws_model_list', $models);
            }
            
            $model_list = get_option("lws_model_list");
            $sender_ids = get_option("lws_sender_id");

            include __DIR__ . '/view/config_template.php';
        }
        //END Model//
        //

        //If user is in senders//
        //
        else if ($active_tab == 'senders'){ 
            //Redirect if not connected
            if (!get_option("lws_user_sms") && !get_option("lws_sender_id")){
                header('Location:' . admin_url( 'admin.php?page=sms-api-settings', 'https' ));
                exit;
            }


            //Add a new sender for the user
            if (isset($_POST['new_sender']) && !empty(sanitize_text_field($_POST['sender_name']))) {
                $sender = sanitize_text_field($_POST['sender_name']);
                // Max 11 chars, alphanumerical and at least a letter (and not empty, ofc) 
                if (strlen($sender) <= 11 && ctype_alnum($sender) && preg_match('/[A-Za-z]/i', $sender)){
                    if (lws_sms_APIAddSender($sender) == "100"){
                        $is_added = true;
                        $add_message = __("Your request for a Sender ID has been acknowledged. Please wait for an operator to validate it.", "lws-sms");
                    }
                    else{
                        $is_added = false;
                        $add_message = __("Your Sender ID must only contain alphanumerical characters and at least 1 letter.", "lws-sms");
                    }
                }
                else{
                    $is_added = false;
                    $add_message = __("Your Sender ID must only contain alphanumerical characters and at least 1 letter.", "lws-sms");
                }
            }
            
            $sender_ids = get_option("lws_sender_id");

            include __DIR__ . "/view/config_senders.php";
        }
        //END Senders//
        //

        //If user is in history//
        //
        else if ($active_tab == 'history'){
            //Redirect if not connected
            if (!get_option("lws_user_sms") && !get_option("lws_sender_id")){
                header('Location:' . admin_url( 'admin.php?page=sms-api-settings', 'https' ));
                exit;
            }

            //Get every SMS (up to 5000) and stock them to use in the page
            $list_history = json_decode(lws_sms_APISMSHistory(), true)['sms_history'];
            $users = array();
            //Get every users with a phone_sms field filled
            foreach (get_users(array('meta_query' => array(array('key' => 'phone_sms')))) as $user){
                //v1
                //Each element in the array is [Name FirstName] => [Phone], obtained using the ID from $user
                //$users[get_user_meta($user->ID, 'last_name', true) . " " . get_user_meta($user->ID, 'first_name', true)] =  get_user_meta($user->ID, 'phone_sms', true);
                
                //v2, the key is now the username
                $users[$user->display_name] =  get_user_meta($user->ID, 'phone_sms', true);
            }
            include __DIR__ . "/view/config_history.php";
        }
        //END History//
        //

        //If user is in the API//
        //
        else if ($active_tab == 'automation'){
            //List of every hooks to have implemented
            $models = get_option('lws_model_list');
            $sender_ids = get_option("lws_sender_id");
            $already_on = false;
            
            //Getting all checkboxes status, creating it if needed of fetching it for the admin side
            $checked_options = array();
            if (!get_option('lws_checked_options')){
                foreach (HOOKS as $hook){
                    $checked_options[$hook]['is_checked'] = false;
                    $checked_options[$hook]['used'] = 'NO';
                }
                foreach (ADMIN_HOOKS as $hook){
                    $checked_options[$hook]['is_checked'] = false;
                    $checked_options[$hook]['used'] = 'NO';
                }
                update_option('lws_checked_options', $checked_options);
            }
            else{
                $checked_options = get_option('lws_checked_options');
            }
            
            //Submitted form phone SMS
            if (isset($_POST['update_phone_sms'])){
                $user = wp_get_current_user();
            	if ( isset($_POST['phone_sms']) && strlen(sanitize_text_field($_POST['phone_sms'])) > 1 && is_numeric(sanitize_text_field($_POST['phone_sms'])) ) {
                    $num = sanitize_text_field($_POST['phone_sms']);
                    if (substr($num, 0 , 1) == "0" && strlen($num) == 10){
                        $num = substr($num, 1);
                    }
                    $num = sanitize_text_field($_POST['countryCode']) . $num;
                    update_user_meta( $user->ID, 'phone_sms', $num );
    	            update_user_meta( $user->ID, 'phone_sms_print', sanitize_text_field($_POST['phone_sms']) );
            	}
                else{
    	            delete_user_meta( $user->ID, 'phone_sms');
    	            delete_user_meta( $user->ID, 'phone_sms_print');
    	        }
    	        $update_sms = true;
            }
            
            $timer_cron = get_option("lws_timer_cron");
            include __DIR__ . '/view/account_checkbox.php';
        }
        //END SMSAPI//
        //

        //If user is in marketing//
        //
        else if ($active_tab == 'campaign'){
            
            //Redirect if not connected
            if (!get_option("lws_user_sms") && !get_option("lws_sender_id")){
                header('Location:' . admin_url( 'admin.php?page=sms-api-settings', 'https' ));
                exit;
            }
            
            $no_balance = false;
            $sms_sent = -1;
            $empty_template = false;
            
            $clients_ok = array();
            if (!$clients = get_option('lws_ads_clients')){
                $clients = array();
                update_option('lws_ads_clients', $clients);
            }
            
            foreach ($clients as $id => $value){
                if ($value){
                    $clients_ok[] = $id;
                }
            }
       
            if (!$models = get_option('lws_model_list')){
                $models = array();
                update_option('lws_model_list', $models);
            }
     
            $client_filter_country = array();
            $clients_SMS = array();
            $returning = array();
            $month_old = array();
            $ten_month_old = array();
            $senders = get_option("lws_sender_id");
            
            global $wpdb;
            //Get country of every customers and their registered time (if < 1 month) for later use
            $customer_lookup = $wpdb->get_results("SELECT * FROM " . $wpdb->prefix . "wc_customer_lookup");
            foreach ($customer_lookup as $customer){
                $client_filter_country[esc_html($customer->customer_id)] = esc_html($customer->country);
                if (time() - strtotime($customer->date_registered) <= 2629743){
                    $month_old[] = $customer->customer_id;
                }
                if (time() - strtotime($customer->date_registered) >= 26297430){
                    $ten_month_old[] = $customer->customer_id;
                }
            }
            
            $order_stats = $wpdb->get_results("SELECT * FROM " . $wpdb->prefix . "wc_order_stats");
            foreach ($order_stats as $order){
                if ($order->returning_customer){
                    $returning[] = $order->customer_id;
                }
            }
            
            //Filters for client >  1 month and returning and > 10 months
            $client_filter_registered_1_month = array();
            $client_filter_registered_10_months = array();
            $client_filter_returning = array();
            foreach ($clients_ok as $client){
                foreach ($month_old as $filter){
                    if ($filter == $client){
                        $client_filter_registered_1_month[] = esc_html($client);
                        break;
                    }
                }
                foreach ($ten_month_old as $filter){
                    if ($filter == $client){
                        $client_filter_registered_10_months[] = esc_html($client);
                        break;
                    }
                }
                
                foreach($returning as $return){
                    if ($return == $client){
                        $client_filter_returning[] = esc_html($client);
                        break;
                    }
                }
            }
            
            
            //If choosing to use a model
            if (isset($_POST['send_sms_market_model'])){
                if (sanitize_text_field($_POST['models']) != "NO"){
                    $model_selected = "";
                    switch(sanitize_text_field($_POST['filter'])){
                        case "country":
                            foreach ($client_filter_country as $client => $value){
                                if (sanitize_text_field($_POST['country_value']) == $value){
                                    $clients_SMS[] = $client;
                                }
                            }
                            break;
                        case "return":
                            $clients_SMS = $client_filter_returning;
                            break;
                        case "month_old":
                            $clients_SMS = $client_filter_registered_1_month;
                            break;
                        case "month_ten":
                            $clients_SMS = $client_filter_registered_10_months;
                            break;
                        default:
                            $clients_SMS = $clients_ok;
                            break;
                    }
                    $sms_sent = 0;
                    foreach ($models as $model){
                        if ($model['name'] == sanitize_text_field($_POST['models'])){
                            $model_selected = $model;
                            if ((json_decode(lws_sms_APIBalance(), true)['solde']) - ($model['nb_sms'] * count($clients_SMS)) >= 0){
                                foreach($clients_SMS as $client){
                                    $user = get_user_by('id', $client);
                                    if ($user->phone_sms){
                                        if ((json_decode(lws_sms_APIBalance(), true)['solde']) - $model['nb_sms'] >= 0){
                                            $message = $model_selected['message'];
                                            $message = str_replace("[[Nom]]",  $user->last_name, $message);
                                            $message = str_replace("[[Prenom]]", $user->first_name, $message);
                                            $message = str_replace(
                                                "[[Adresse]]", 
                                                $user->shipping_address_1 . ', ' . 
                                                $user->shipping_address_2 . ' ' .
                                                $user->shipping_city      . ', ' .
                                                $user->shipping_postcode     . ' ' .
                                                $user->shipping_country, $message);
                                            $message = str_replace("[[NomSite]]", get_bloginfo('name'), $message);
                                            $message = str_replace("[[URLSite]]", get_bloginfo('url'), $message);
                                            $message = str_replace("[[Date]]",gmdate("[Y-m-d] - H:i:s : ", time()) , $message);
                                            lws_sms_APISendSMS($model_selected['sender'], $user->phone_sms, $message);
                                            $sms_sent += 1;
                                        }
                                        else{
                                            $no_balance = true;
                                            break;
                                        }
                                    }
                                }
                                $sms_sent = true;
                                break;
                            }
                        }
                    }
                }
                else{
                    $empty_template = true;
                }
            }
            //If choosing to write the SMS
            else if (isset($_POST['send_sms_market'])){
                switch(sanitize_text_field($_POST['filter'])){
                    case "country":
                        foreach ($client_filter_country as $client => $value){
                            if (sanitize_text_field($_POST['country_value']) == $value){
                                $clients_SMS[] = $client;
                            }
                        }
                        break;
                    case "return":
                        $clients_SMS = $client_filter_returning;
                        break;
                    case "month_old":
                        $clients_SMS = $client_filter_registered_1_month;
                        break;
                    case "month_ten":
                        $clients_SMS = $client_filter_registered_10_months;
                        break;
                    default:
                        $clients_SMS = $clients_ok;
                        break;
                }
                $sms_sent = 0;
                if ((json_decode(lws_sms_APIBalance(), true)['solde']) - ($_POST['number_message'] * count($clients_SMS)) >= 0){
                    foreach($clients_SMS as $client){
                        $user = get_user_by('id', $client);
                        if ($user->phone_sms){
                            if ((json_decode(lws_sms_APIBalance(), true)['solde']) - sanitize_text_field($_POST['number_message']) >= 0){
                                $message = sanitize_textarea_field($_POST['marketing_model']);
                                $message = str_replace("[[Nom]]",  $user->last_name, $message);
                                $message = str_replace("[[Prenom]]", $user->first_name, $message);
                                $message = str_replace(
                                    "[[Adresse]]", 
                                    $user->shipping_address_1 . ', ' . 
                                    $user->shipping_address_2 . ' ' .
                                    $user->shipping_city      . ', ' .
                                    $user->shipping_postcode     . ' ' .
                                    $user->shipping_country, $message);
                                $message = str_replace("[[NomSite]]", get_bloginfo('name'), $message);
                                $message = str_replace("[[URLSite]]", get_bloginfo('url'), $message);
                                $message = str_replace("[[Date]]",gmdate("[Y-m-d] - H:i:s : ", time()) , $message);
                                $message = trim($message);
                                lws_sms_APISendSMS(sanitize_text_field($_POST['senders']), $user->phone_sms, $message);
                                $sms_sent += 1;
                            }
                            else{
                                $no_balance = true;
                                break;
                            }                        
                        }
                    }
                }
            }
            
            $model_list = wp_json_encode($models, JSON_PRETTY_PRINT);
            $SMSBalance = json_decode(lws_sms_APIBalance(), true)['solde'];
            include __DIR__ . "/view/config_marketing.php";
        }
        

        //END Marketing//
        //
    }   
}

function lws_sms_requiresUnicodeEncoding($content){
    $gsmCodePoints = array_map(
    lws_sms_convertIntoUnicode(),
    [
    '@', '£', '$', '¥', 'è', 'é', 'ù', 'ì', 'ò', 'ç', "\r", 'Ø', 'ø', "\n", 'Å', 'å',
    'Δ', '_', 'Φ', 'Γ', 'Λ', 'Ω', 'Π', 'Ψ', 'Σ', 'Θ', 'Ξ', 'Æ', 'æ', 'ß', 'É',
    ' ', '!', '"', '#', '¤', '%', '&', '\'', '(', ')', '*', '+', ',', '-', '.', '/',
    '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', ':', ';', '<', '=', '>', '?',
    '¡', 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O',
    'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z', 'Ä', 'Ö', 'Ñ', 'Ü', '§',
    '¿', 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o',
    'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z', 'ä', 'ö', 'ñ', 'ü', 'à',
    "\f", '^', '{', '}', '\\', '[', '~', ']', '|', '€',
    ]
);

// Split $text into an array in a way that respects multibyte characters.
$textChars = preg_split('//u', $content, null, PREG_SPLIT_NO_EMPTY);

// Array of codepoint values for characters in $text.
$textCodePoints = array_map(lws_sms_convertIntoUnicode(), $textChars);

// Filter the array to contain only codepoints from $text that are not in the set of valid GSM codepoints.
$nonGsmCodePoints = array_diff($textCodePoints, $gsmCodePoints);

// The text contains unicode if the result is not empty.
return !empty($nonGsmCodePoints);
}

function lws_sms_convertIntoUnicode(){
    return function ($char) {
    $k = mb_convert_encoding($char, 'UTF-16LE', 'UTF-8');
    $k1 = ord(substr($k, 0, 1));
    $k2 = ord(substr($k, 1, 1));

    return $k2 * 256 + $k1;
    };
}

add_action( 'wp_ajax_unicodeChecker', 'lws_sms_ajax_unicode' );
function lws_sms_ajax_unicode(){
    if(isset($_POST['ajax_text'])){
        $text = sanitize_textarea_field($_POST['ajax_text']);
        echo esc_html(lws_sms_requiresUnicodeEncoding($text));
    }
    wp_die();
}

add_action( 'wp_ajax_automation_update', 'lws_sms_ajax_automation' );
function lws_sms_ajax_automation(){
    
    $models = get_option('lws_model_list');
    $already_on = false;
    $checked_options = get_option('lws_checked_options');
    foreach ($_POST as $P){
        $POST[] = sanitize_text_field($P);
    }
    

    $hook_info = explode("|", $POST[1]);
    $hook = $hook_info[0];
    if (substr($hook, 0, 5) == "admin"){
        $id_hook = "A|" . $hook_info[1];
    }
    else{
        $id_hook = $hook_info[1];
    }

    //When submitted ()
    if ($POST[3] == "true" && $POST[2] != "NO"){
        $select = $POST[2];
        foreach ($models as $key => $model){
            //Stock the choosen model for later
            if ($model['name'] == $select){
                $model_active = $model;
                $model_active_key = $key;
                //If for some reason there is nothing in ['in_use'],
                //create a new entry with 'NO'
                if (count($model['in_use']) == 0){
                    $model['in_use'][] = 'NO';
                }
            }
            foreach($model['in_use'] as $key2 => $used){
                //Check every models and delete any tags already on
                if ($used == $hook){
                    if (count($model['in_use']) == 1){
                        $models[$key]['in_use'][$key2] = 'NO';
                    }
                    else{
                        unset($models[$key]['in_use'][$key2]);
                    }
                }
            }
        }
        //Take the saved model and check his tags
        foreach ($model_active['in_use'] as $key2 => $used){
            //Delete any 'NO' tags
            if ($used == 'NO'){
                unset($models[$model_active_key]['in_use'][$key2]);
            }
            //Add the tag if it is not already set
            if (!$already_on){
                $models[$model_active_key]['in_use'][$id_hook] = $hook;
                $already_on = true;
                update_option("lws_model_list", $models);
            }
        }
        //Update the checkboxes checker to save the new values
        $checked_options[$hook]['is_checked'] = true;
        $checked_options[$hook]['used'] = $model_active['id'];
        if ($hook == 'sessions' && ($POST[4] != "NO")){
            wp_clear_scheduled_hook('check_sessions_cron');
            if (!wp_next_scheduled ('check_sessions_cron') && $POST[4] != "NO"){
                update_option("lws_timer_cron", $POST[4]);
                $rep = wp_schedule_event(time(), "twicedaily", 'check_sessions_cron');
            }
        }
        if ($hook == 'dailies'){
            wp_clear_scheduled_hook('daily_logs_cron');
            if (!wp_next_scheduled ('daily_logs_cron')){
                $rep = wp_schedule_event(time(), "daily", 'daily_logs_cron');
            }
        }
        //1 == checkbox activée
        echo esc_html(1);
    }

    //If the checkbox is unchecked, update the checker
    else{
        $checked_options[$hook]['is_checked'] = false;
        $checked_options[$hook]['used'] = 'NO';
        if ($hook == 'sessions'){
            $rep = wp_clear_scheduled_hook('check_sessions_cron');
            delete_option("lws_timer_cron");
        }
        if ($hook == 'dailies'){
            wp_clear_scheduled_hook('daily_logs_cron');
        }

        foreach ($models as $key => $model){
            foreach ($model['in_use'] as $key2 => $used){
                if ($used == $hook){
                    if (count($model['in_use']) <= 1){
                        unset($models[$key]['in_use'][$id_hook]);
                        $models[$key]['in_use'][] = "NO";
                    }
                    else{
                        unset($models[$key]['in_use'][$id_hook]);
                    }
                    break;
                }
            }
            update_option("lws_model_list", $models);
            echo esc_html(-1);
            break;
        }
    }

    update_option('lws_checked_options', $checked_options);
    wp_die();
}


//API CALLS//

//Connect to API to get infos about user
function lws_sms_APIInfo($username, $apikey){
    $action = "check-client";
    return wp_remote_post("https://sms.lws.fr/plugin/api", array(
        'method'      => 'POST',
        'timeout'     => 45,
        'redirection' => 5,
        'httpversion' => '1.0',
        'blocking'    => true,
        'headers'     => array(),
        'body'        => array(
            'action' => $action,
            'username' => urlencode($username),
            'api_key_client' => $apikey
            ),
        'cookies'     => array()
        )
    );
}

//Send a SMS to $receiver by $sender with the content $message passed in parameters
//$receiver has to be a number with the country code, ex: 336XXXXXXXX for France
function lws_sms_APISendSMS($sender, $receiver, $message){
    $client = (get_option("lws_user_sms"));
    $action = "send-sms";
    //return wp_remote_get("https://sms.lws.fr/sms/api?action=" . $action . "&to=" . $receiver . "&from=" . $sender . "&sms=" . $message . "&api_key=" . $client['api_key']);
    return wp_remote_post("https://sms.lws.fr/sms/api", array(
        'method'      => 'POST',
        'timeout'     => 45,
        'redirection' => 5,
        'httpversion' => '1.0',
        'blocking'    => true,
        'headers'     => array(),
        'body'        => array(
            'action' => $action,
            'to' => $receiver,
            'from' => $sender,
            'sms' => urlencode($message),
            'api_key' => $client['api_key']
        ),
        'cookies'     => array()
        )
    );
}

//Get how many remaining SMS $client has
function lws_sms_APIBalance(){
    $client = (get_option("lws_user_sms"));
    $action = "solde-client";
    return wp_remote_post("https://sms.lws.fr/plugin/api", array(
        'method'      => 'POST',
        'timeout'     => 45,
        'redirection' => 5,
        'httpversion' => '1.0',
        'blocking'    => true,
        'headers'     => array(),
        'body'        => array(
            'action' => $action,
            'id_client' => urlencode($client['id_client']),
            'api_key_client' => $client['api_key'],
            ),
        'cookies'     => array()
        )
    )['body'];
}

//Check if the client's API KEY is still valid
//Returns directly the code, not $response
function lws_sms_APICheckAPI(){
    $client = (get_option("lws_user_sms"));
    $action = "check-api-key";
    return json_decode(wp_remote_post("https://sms.lws.fr/plugin/api", array(
        'method'      => 'POST',
        'timeout'     => 45,
        'redirection' => 5,
        'httpversion' => '1.0',
        'blocking'    => true,
        'headers'     => array(),
        'body'        => array(
            'action' => $action,
            'id_client' => urlencode($client['id_client']),
            'api_key_client' => $client['api_key'],
            ),
        'cookies'     => array()
        )
    )['body'], true)['code'];
}

//Get every SenderID for $client
function lws_sms_APILoadSender(){
    $client = (get_option("lws_user_sms"));
    $action = "load-sender";
    return wp_remote_post("https://sms.lws.fr/plugin/api", array(
        'method'      => 'POST',
        'timeout'     => 45,
        'redirection' => 5,
        'httpversion' => '1.0',
        'blocking'    => true,
        'headers'     => array(),
        'body'        => array(
            'action' => $action,
            'id_client' => urlencode($client['id_client']),
            ),
        'cookies'     => array()
        )
    )['body'];
}

//Get every SMS (with a limit of 5000) sent by $client
function lws_sms_APISMSHistory(){
    $client = (get_option("lws_user_sms"));
    $action = "get-sms-history";
    return wp_remote_post("https://sms.lws.fr/plugin/api", array(
        'method'      => 'POST',
        'timeout'     => 45,
        'redirection' => 5,
        'httpversion' => '1.0',
        'blocking'    => true,
        'headers'     => array(),
        'body'        => array(
            'action' => $action,
            'id_client' => urlencode($client['id_client']),
            ),
        'cookies'     => array()
        )
    )['body'];
}

//Ask to add $ID as a new sender for $client
function lws_sms_APIAddSender($ID){
    $client = (get_option("lws_user_sms"));
    $action = "add-sender-wp";
    return json_decode(wp_remote_post("https://sms.lws.fr/plugin/api", array(
        'method'      => 'POST',
        'timeout'     => 45,
        'redirection' => 5,
        'httpversion' => '1.0',
        'blocking'    => true,
        'headers'     => array(),
        'body'        => array(
            'action' => $action,
            'sender' => $ID,
            'id_client' => urlencode($client['id_client']),
            ),
        'cookies'     => array()
        )
    )['body'], true)['code'];
}

//END API CALLS//

    //WOOCOMMERCE//

//ADD EVERY FIELDS NECESSARY IN WOOCOMMERCE PAGES//

//Add a "Alert me" button on a out ok stock product
add_filter('woocommerce_product_meta_start', 'lws_sms_add_oos_button', 20);
function lws_sms_add_oos_button(){
    //Get the actual product
    global $product;

    //If client want alerts, gets product id & user id
    //Add them in user's metadata for this product
    //Schedule a cron if not already scheduled for this user/product
    if (isset($_POST['alert_stock'])){
        $args = array('product' => $product, 'user' => wp_get_current_user());
        update_user_meta(wp_get_current_user()->ID, 'alert_stock_' . $product->ID, true);
        if (! wp_next_scheduled ( 'check_stock_alert_cron', $args )) {
            $response = wp_schedule_event( time(), 'hourly', 'check_stock_alert_cron', $args );
        }
    }

    // Check if the client is okay with receiving SMS for stock
    // and admin activated the option
    $admin_ok = get_option('lws_checked_options');
    $checkboxes = array();
    if (!get_user_meta(wp_get_current_user()->ID, 'checkboxes', true)){
        update_user_meta(wp_get_current_user()->ID, 'checkboxes', array());
    }
    else {
        $checkboxes = get_user_meta(wp_get_current_user()->ID, 'checkboxes', true);
    }
    $client_ok = false;
    if ($admin_ok['stock']['is_checked']){
        if (wp_get_current_user()->phone_sms){
            if ($checkboxes['stock']['is_checked'] || count($checkboxes) == 0){
                $client_ok = true;
            }
        }
    }
    
    //If the product is out of stock & client + admin are okay with SMS here,
    //show the button to alert clients but disable it if client already is alerted
    ?>
        <?php if ($product->stock_status == 'outofstock' && $client_ok): ?>
            <form method="post">
            <?php if (get_user_meta(wp_get_current_user()->ID, 'alert_stock_' . $product->ID, true)) :?>
                    <input disabled name="alert_stock" type="submit" id='alert_stock' value="<?php esc_attr_e("Send alert when product back in stock", "lws-sms")?>">
            <?php else : ?>
                    <input name="alert_stock" type="submit" id='alert_stock' value="<?php esc_attr_e("Send alert when product back in stock", "lws-sms")?>">
            <?php endif ?>
            </form> 
        <?php endif ?>
<?php
}

//Add fields to the My Account edition form of WC
add_action("woocommerce_edit_account_form", "lws_sms_add_fields");
function lws_sms_add_fields(){
    $user = wp_get_current_user();
    ?>
    <p class="woocommerce-form-row woocommerce-form-row--wide form-row form-row-wide">
		<label for="phone_sms"><?php esc_html_e( 'Phone number', "lws-sms" ); ?></label>
        <?php include __DIR__ . "/php/select_country.php";?>
        <input required class="woocommerce-Input woocommerce-Input--text input-text input_phone_sms" type="tel" id="phone_sms" name="phone_sms" value="<?php echo esc_attr( $user->phone_sms_print );?>">
        <br>
        <small style="font-size: 11px;"><?php esc_html_e('Allow access to SMS marketing such as alerts when a product is in stock.', "lws-sms"); ?>
        <br>
        <?php esc_html_e('Do not add the country code to your number.', "lws-sms");?></small> <br>
        <?php $okay_ad = get_option('lws_ads_clients');
        if (!$okay_ad) { 
            $okay_ad = array($user->ID => false);
            update_option( 'lws_ads_clients', $okay_ad, true);
        }
        ?>
        <br>
        <label for="checkboxes_ads"><?php esc_html_e("I agree to receive marketing communications by SMS","lws-sms")?>
        <input type="checkbox" id="checkboxes_ads"  name="checkboxes_ads" <?php echo $okay_ad[$user->ID] ? esc_attr('checked') : '';?>>
        </label>

        <script>
            document.querySelector("#checkboxes_ads").addEventListener("change", function() {
                if (this.checked){
                    jQuery("#phone_sms").prop("required", true);
                }
                else{
                    jQuery("#phone_sms").prop("required", false);
                }
            });
        </script>
    </p>
<?php
}

//Save the SMS Phone Field from the My Account edition form of WC
add_action( 'woocommerce_save_account_details', 'lws_sms_save_phone_number' );
function lws_sms_save_phone_number( $user_id ) { 
    update_user_meta( $user_id, 'checkboxes', array());
	if ( isset($_POST['phone_sms']) && strlen((sanitize_text_field($_POST['phone_sms']))) > 1 && is_numeric((sanitize_text_field($_POST['phone_sms'])))) {
        $num = (sanitize_text_field($_POST['phone_sms']));
        if (substr($num, 0 , 1) == "0" && strlen($num) == 10){
            $num = substr($num, 1);
        }
        $num = sanitize_text_field($_POST['countryCode']) . $num;
        $client_ok_ads = get_option('lws_ads_clients');
        if (!$client_ok_ads) { 
            $client_ok_ads = array($user_id => false);
            update_option( 'lws_ads_clients', $client_ok_ads, true);
        }
        if (sanitize_text_field($_POST['checkboxes_ads']) == 'on'){
            $client_ok_ads[$user_id] = true;
            update_option( 'lws_ads_clients', $client_ok_ads, true);
        }
        else{
            $client_ok_ads[$user_id] = false;
            update_option( 'lws_ads_clients', $client_ok_ads, true);
        }

		update_user_meta( $user_id, 'phone_sms', $num );
		update_user_meta( $user_id, 'phone_sms_print', sanitize_text_field($_POST['phone_sms']) );
	}
	else{
	    delete_user_meta( $user_id, 'phone_sms');
	    delete_user_meta( $user_id, 'phone_sms_print');
	}
}

//Add a SMS Phone Field to the Register form of WC 
add_action( 'woocommerce_register_form', 'lws_sms_add_phone_number_to_register');
function lws_sms_add_phone_number_to_register(){
    ?>
    <p class="woocommerce-form-row woocommerce-form-row--wide form-row form-row-wide">
        <label for="phone_sms"><?php esc_html_e( 'Phone number', "lws-sms" ); ?></label>
        <?php include __DIR__ . "/php/select_country.php";?>
        <input required class="woocommerce-Input woocommerce-Input--text input-text input_phone_sms" type="tel" id="phone_sms" name="phone_sms" value="">
        <br>
        <small style="font-size: 11px;"><?php esc_html_e('Allow access to SMS marketing such as alerts when a product is in stock.', "lws-sms"); ?>
        <br>
        <?php esc_html_e('Do not add the country code to your number.', "lws-sms");?></small> <br>
        <label for="checkboxes_ads"><?php esc_html_e("I agree to receive marketing communications by SMS","lws-sms")?>
        <input type="checkbox" id="checkboxes_ads"  name="checkboxes_ads" checked>
        </label>

        <script>
            document.querySelector("#checkboxes_ads").addEventListener("change", function() {
                if (this.checked){
                    jQuery("#phone_sms").prop("required", true);
                }
                else{
                    jQuery("#phone_sms").prop("required", false);
                }
            });
        </script>
    </p>
<?php
}

//Save the SMS Phone Field of the new created user
add_action('woocommerce_created_customer', 'lws_sms_add_phone_sms_customer');
function lws_sms_add_phone_sms_customer($customer_id){
    update_user_meta( $customer_id, 'checkboxes', array());
	if ( isset($_POST['phone_sms']) && strlen(sanitize_text_field($_POST['phone_sms'])) > 1 && is_numeric(sanitize_text_field($_POST['phone_sms']))) {
        $num = sanitize_text_field($_POST['phone_sms']);
        if (substr($num, 0 , 1) == "0" && strlen($num) == 10){
            $num = substr($num, 1);
        }
        $num = sanitize_text_field($_POST['countryCode']) . $num;
        $client_ok_ads = get_option('lws_ads_clients');
        if (!$client_ok_ads) { 
            $client_ok_ads = array($customer_id => false);
            update_option( 'lws_ads_clients', $client_ok_ads, true);
        }
        if (sanitize_text_field($_POST['checkboxes_ads']) == 'on'){
            $client_ok_ads[$customer_id] = true;
            update_option( 'lws_ads_clients', $client_ok_ads, true);
        }
        else{
            $client_ok_ads[$customer_id] = false;
            update_option( 'lws_ads_clients', $client_ok_ads, true);
        }
		update_user_meta( $customer_id, 'phone_sms', $num );
		update_user_meta( $customer_id, 'phone_sms_print', sanitize_text_field($_POST['phone_sms']) );
	}
}

//Add the usual field to the Register form in the checkout page
add_action('woocommerce_after_checkout_registration_form', 'lws_sms_register_checkout_phone_sms');
function lws_sms_register_checkout_phone_sms(){
    ?>
    <p id="phone_sms_checkout" class="woocommerce-form-row woocommerce-form-row--wide form-row form-row-wide"></p>
    <script> 
    if (!!document.getElementById("createaccount")){
        var check = document.getElementById("createaccount")
        check.addEventListener('change', function() {
            if (this.checked) {
                document.getElementById('phone_sms_checkout').innerHTML = `
                <label for="phone_sms"><?php esc_html_e( 'Phone number', "lws-sms" ); ?></label> 
                <?php include __DIR__ . "/php/select_country.php";?>
                <input required class="woocommerce-Input woocommerce-Input--text input-text input_phone_sms" type="tel" id="phone_sms" name="phone_sms" value="">
                <br>
                <small style="font-size: 11px;"><?php esc_html_e('Allow access to SMS marketing such as alerts when a product is in stock.', "lws-sms"); ?>
                <br>
                <?php esc_html_e('Do not add the country code to your number.', "lws-sms");?></small> <br>                
                <label for="checkboxes_ads"><?php esc_html_e("I agree to receive marketing communications by SMS","lws-sms")?>
                <input type="checkbox" id="checkboxes_ads"  name="checkboxes_ads" checked>
                </label>
                <script>
                    document.querySelector("#checkboxes_ads").addEventListener("change", function() {
                        if (this.checked){
                            jQuery("#phone_sms").prop("required", true);
                        }
                        else{
                            jQuery("#phone_sms").prop("required", false);
                        }
                    });<//script>`
            }
            else{
                document.getElementById('phone_sms_checkout').innerHTML = ""
            }
        })
    }
    else{
        document.getElementById('phone_sms_checkout').innerHTML = `
            <label for="phone_sms"><?php esc_html_e( 'Phone number', "lws-sms" ); ?></label> 
            <?php include __DIR__ . "/php/select_country.php";?>
            <input required class="woocommerce-Input woocommerce-Input--text input-text input_phone_sms" type="tel" id="phone_sms" name="phone_sms" value="">
            <br>
            <small style="font-size: 11px;"><?php esc_html_e('Allow access to SMS marketing such as alerts when a product is in stock.', "lws-sms");?>
            <br> 
            <?php esc_html_e('Do not add the country code to your number.', "lws-sms");?></small> <br>
            <label for="checkboxes_ads"><?php esc_html_e("I agree to receive marketing communications by SMS","lws-sms")?>
            <input type="checkbox" id="checkboxes_ads"  name="checkboxes_ads" checked>
            </label>
            <script>
                document.querySelector("#checkboxes_ads").addEventListener("change", function() {
                    if (this.checked){
                        jQuery("#phone_sms").prop("required", true);
                    }
                    else{
                        jQuery("#phone_sms").prop("required", false);
                    }
                });
            <//script>
        `
    }
    </script>
<?php
}

//Save the fields when the user is created
add_action( 'user_register', 'lws_sms_myplugin_registration_save', 10, 1 );
function lws_sms_myplugin_registration_save( $user_id ) {
    if ( isset($_POST['phone_sms']) && strlen(sanitize_text_field($_POST['phone_sms'])) > 1 && is_numeric(sanitize_text_field($_POST['phone_sms']))){
    
        $num = sanitize_text_field($_POST['phone_sms']);
        if (substr($num, 0 , 1) == "0" && strlen($num) == 10){
            $num = substr($num, 1);
        }
        $num = sanitize_text_field($_POST['countryCode']) . $num;
        update_user_meta( $user_id, 'phone_sms', $num ); 
        update_user_meta( $user_id, 'phone_sms_print', sanitize_text_field($_POST['phone_sms']) );
        //Getting all checkboxes status, creating it if needed of fetching it for the client side
        $checked_options_client = array();
        if (!get_option('lws_checked_options_client')){
            foreach (HOOKS as $hook){
                $checked_options_client[$hook]['is_checked'] = true;
            }
            update_option('lws_checked_options_client', $checked_options_client);
        }
        $client_ok_ads = get_option('lws_ads_clients');
        if (!$client_ok_ads) { 
            $client_ok_ads = array($user_id => false);
            update_option( 'lws_ads_clients', $client_ok_ads, true);
        }
        if (sanitize_text_field($_POST['checkboxes_ads']) == 'on'){
            $client_ok_ads[$user_id] = true;
            update_option( 'lws_ads_clients', $client_ok_ads, true);
        }
        else{
            $client_ok_ads[$user_id] = false;
            update_option( 'lws_ads_clients', $client_ok_ads, true);
        }
    }
}

//END FIELDS//

//SMS SCENARIOS//

//Main function of the segment. Send a SMS with the given parameters
function lws_sms_send_SMS($order = null, $user, $usage, $num, $coupon = null, $product = null){
    $models = get_option('lws_model_list');
    foreach ($models as $model){
            foreach($model['in_use'] as $in_use){
                if ($in_use == $usage){
                    if ((json_decode(lws_sms_APIBalance(), true)['solde']) - $model['nb_sms'] >= 0){
                        $message = $model['message'];
                        $message = str_replace("[[Nom]]",  $user->last_name, $message);
                        $message = str_replace("[[Prenom]]", $user->first_name, $message);
                        if ($order){
                            $message = str_replace("[[Prix]]",  $order->price, $message);
                            $message = str_replace("[[NumCmde]]", $order->ID, $message);
                        }
                        else{
                            $message = str_replace("[[Prix]]",  "", $message);
                            $message = str_replace("[[NumCmde]]", "", $message);
                        }

                        $message = str_replace(
                            "[[Adresse]]", 
                            $user->shipping_address_1 . ', ' . 
                            $user->shipping_address_2 . ' ' .
                            $user->shipping_city      . ', ' .
                            $user->shipping_postcode     . ' ' .
                            $user->shipping_country, $message);
                        if ($coupon){
                            $message = str_replace("[[CodeCoupon]]", $coupon->code, $message);
                            $message = str_replace("[[DescriptionCoupon]]", $coupon->get_description(), $message);
                            $message = str_replace("[[ValeurCoupon]]", $coupon->amount, $message);
                        }
                        else{
                            $message = str_replace("[[CodeCoupon]]", "", $message);
                            $message = str_replace("[[DescriptionCoupon]]", "", $message);
                            $message = str_replace("[[ValeurCoupon]]", "", $message);
                        }
                        
                        if ($product){
                            $message = str_replace("[[NomProduit]]", $product->name, $message);
                            $message = str_replace("[[PrixProduit]]", $product->price, $message);
                            $message = str_replace("[[DescriptionProduit]]", $product->description, $message);
                        }
                        else{
                            $message = str_replace("[[NomProduit]]", "", $message);
                            $message = str_replace("[[PrixProduit]]", "", $message);
                            $message = str_replace("[[DescriptionProduit]]", "", $message);
                        }
                        
                        $message = str_replace("[[PanierAbandonnes]]",  "", $message);
                        $message = str_replace("[[MontantTotal]]",  "", $message);
                        $message = str_replace("[[Commandes]]",  "", $message);
                        $message = str_replace("[[NouveauxClients]]",  "", $message);
                        $message = str_replace("[[MeilleureVente]]",  "", $message);
                        $message = str_replace("[[ListePanier]]", "", $message);

                        
                        $message = str_replace("[[NomSite]]", get_bloginfo('name'), $message);
                        $message = str_replace("[[URLSite]]", get_bloginfo('url'), $message);
                        $message = str_replace("[[Date]]",gmdate("[Y-m-d] - H:i:s : ", time()) , $message);
                        lws_sms_APISendSMS($model['sender'], $num, $message);
                        break;
                }
            }
        }
    }
}

//Send a SMS to the user when the product they wanted is in stock
add_action('check_stock_alert_cron', 'lws_sms_check_stock_alert', 10, 2);
function lws_sms_check_stock_alert($product, $user){
    $usage = 'stock';
    if ($product->stock_status == 'instock'){
        if( $user ){
            //Only if admin has activated it first
            $admin_check = get_option('lws_checked_options');
            if ($admin_check[$usage]['is_checked']){
                //Only if client has authorized SMS for this
                $checkboxes = array();
                get_user_meta( $user->ID, 'checkboxes', true) ? 
                $checkboxes = get_user_meta( $user->ID, 'checkboxes', true) : update_user_meta($user->ID, 'checkboxes', $checkboxes);
                    
                if (count($checkboxes) == 0 || $checkboxes[$usage]['is_checked']){
                    lws_sms_send_SMS(null, $user, $usage, $user->phone_sms, null, $product);
                }

            } 
        }
        $args = array('product' => $product, 'user' => $user);
        wp_clear_scheduled_hook( 'check_stock_alert_cron', $args);
        delete_user_meta($user->ID, 'alert_stock_' . $product->ID);
    }
}

//Send a SMS to the order's owner when their order's status is "ongoing"
add_action( 'woocommerce_order_status_processing', 'lws_sms_send_mail_payment_processing' );
function lws_sms_send_mail_payment_processing( $order_id ){
    $order = wc_get_order( $order_id );
    $user = $order->get_user();
    $usage = 'processing';
    if( $user ){
        //Only if admin has activated it first
        $admin_check = get_option('lws_checked_options');
        if ($admin_check[$usage]['is_checked']){
            //Only if client has authorized SMS for this
            $checkboxes = array();
            get_user_meta( $user->ID, 'checkboxes', true) ? 
            $checkboxes = get_user_meta( $user->ID, 'checkboxes', true) : update_user_meta($user->ID, 'checkboxes', $checkboxes);
            
            if (count($checkboxes) == 0 || $checkboxes[$usage]['is_checked']){
                lws_sms_send_SMS($order, $user, $usage, $user->phone_sms, null, null);
            }
        } 
    }
}

//Send a SMS to the order's owner when their order's status is "cancelled"
add_action( 'woocommerce_order_status_cancelled', 'lws_sms_send_mail_payment_cancelled' );
function lws_sms_send_mail_payment_cancelled( $order_id ){
   $order = wc_get_order( $order_id );
    $user = $order->get_user();
    $usage = 'cancelled';
    if( $user ){
        //Only if admin has activated it first
        $admin_check = get_option('lws_checked_options');
        if ($admin_check[$usage]['is_checked']){
            //Only if client has authorized SMS for this
            $checkboxes = array();
            get_user_meta( $user->ID, 'checkboxes', true) ? 
            $checkboxes = get_user_meta( $user->ID, 'checkboxes', true) : update_user_meta($user->ID, 'checkboxes', $checkboxes);
            
            if (count($checkboxes) == 0 || $checkboxes[$usage]['is_checked']){
                lws_sms_send_SMS($order, $user, $usage, $user->phone_sms, null, null);
            }
        } 
    }
}

//Send a SMS to the order's owner when their order's status is "completed"
add_action( 'woocommerce_order_status_completed', 'lws_sms_send_mail_payment_complete' );
function lws_sms_send_mail_payment_complete( $order_id ){
    $order = wc_get_order( $order_id );
    $user = $order->get_user();
    $usage = 'completed';
    if( $user ){
        //Only if admin has activated it first
        $admin_check = get_option('lws_checked_options');
        if ($admin_check[$usage]['is_checked']){
            //Only if client has authorized SMS for this
            $checkboxes = array();
            get_user_meta( $user->ID, 'checkboxes', true) ? 
            $checkboxes = get_user_meta( $user->ID, 'checkboxes', true) : update_user_meta($user->ID, 'checkboxes', $checkboxes);
            
            if (count($checkboxes) == 0 || $checkboxes[$usage]['is_checked']){
                lws_sms_send_SMS($order, $user, $usage, $user->phone_sms, null, null);
            }
        } 
    } 
}

//Send a SMS to every client getting ads to inform them of a new coupon
add_action( 'woocommerce_update_coupon', 'lws_sms_after_new_coupon_created', 10, 1 );
function lws_sms_after_new_coupon_created( $coupon_id ){
    $users = array();
    if (!$users = get_option('lws_ads_clients')){
        $users = array(wp_get_current_user() => false);
        update_option('lws_ads_clients', $users);
    }
    
    $usage = 'coupon';
    global $woocommerce;
    $coupon = new WC_Coupon($coupon_id);
    //Only if admin has activated it first
    $admin_check = get_option('lws_checked_options');
    if ($admin_check[$usage]['is_checked']){
        //Only if client has authorized SMS for this
        foreach ($users as $user_id => $ok){
            if ($ok){
                $user = get_user_by('id', $user_id);
                if ($user->phone_sms){
                    lws_sms_send_SMS(null, $user, $usage, $user->phone_sms, $coupon, null);
                }
            }
        }
    } 
}

//SMS to the admin if new order
add_action('woocommerce_new_order', 'lws_sms_new_order', 10, 1);
function lws_sms_new_order($order_id){
    $users = get_users();
    $users_can = array();
    $order = wc_get_order( $order_id );
    $client = $order->get_user();
    $usage = 'admin_new_order';
    $admin_check = get_option('lws_checked_options');
    if ($admin_check[$usage]['is_checked']){
        foreach ( $users as $user ) {
            if (user_can($user, 'manage_woocommerce')){
                if ($user->phone_sms){
                    lws_sms_send_SMS($order, $client, $usage, $user->phone_sms, null, null);
                }
            }
        }
    }


    
}

add_action('check_sessions_cron', 'lws_sms_check_sessions');
function lws_sms_check_sessions(){
    $usage = 'sessions';
    $admin_check = get_option('lws_checked_options');
    $timer = get_option("lws_timer_cron");
    $time = 0;
    //Based on a 48h-expiry
    switch ($timer) {
        case "hour":
            $time = 169200;
            break;
        case "three":
            $time = 162000;
            break;
        case "six":
            $time = 151200;
            break;
        case "twelve":
            $time = 129600;
            break;
        case "twentyfour":
            $time = 86400;
            break;
    }

    if ($admin_check[$usage]['is_checked']){
        //Get every sessions stocked
        global $wpdb;
        $order_sessions = $wpdb->get_results("SELECT * FROM ". $wpdb->prefix."woocommerce_sessions");
        
        foreach ($order_sessions as $session){
            //Get everything known about the user
            $customer = unserialize(unserialize($session->session_value)['customer']);
            $user = get_user_by('id', $customer['id']);

            $checkboxes = array();
            get_user_meta( $user->ID, 'checkboxes', true) ? 
            $checkboxes = get_user_meta( $user->ID, 'checkboxes', true) : update_user_meta($user->ID, 'checkboxes', $checkboxes);

            if (count($checkboxes) == 0 || $checkboxes[$usage]['is_checked']){
                
                //Get the phone_sms of an existing customer, if exist
                $num = get_user_meta($customer['id'], 'phone_sms', true);
                
                //Check expiry of the session
                $expire = ($session->session_expiry);
                
                //Get their cart if any
                $cart = unserialize(unserialize($session->session_value)['cart']);
                
                //Get the total amount of the cart
                $total = unserialize(unserialize($session->session_value)['cart_totals'])['total'];
                
                //Get every applied coupons on the cart
                $coupons = unserialize(unserialize($session->session_value)['applied_coupons']);
                
                $list_coupon = "";
                foreach ($coupons as $coupon){
                    $listcoupon = $list_coupon . $coupon . "&nbsp;";
                }
                
                $list_product = "";
                foreach ($cart as $c){
                    $p = wc_get_product($c['product_id'])->get_data();
                    $list_product = $list_product . $p['name'] . "(" . $c['quantity'] . ") ; " ;
                }

                if ($expire - time() <= $time && $expire - time() > 0){
                    if($num && !empty($cart)){
                        $models = get_option('lws_model_list');
                        foreach ($models as $model){
                            if ((json_decode(lws_sms_APIBalance(), true)['solde']) - $model['nb_sms'] >= 0){
                                foreach($model['in_use'] as $in_use){
                                    if ($in_use == $usage){
                                        $message = $model['message'];
                                        $message = str_replace("[[Nom]]",  $user->last_name, $message);
                                        $message = str_replace("[[Prenom]]", $user->first_name, $message);

                                        $message = str_replace("[[MontantTotal]]", $total, $message);
                                        $message = str_replace("[[ListePanier]]", $list_product, $message);

                                        $message = str_replace("[[NomSite]]", get_bloginfo('name'), $message);
                                        $message = str_replace("[[URLSite]]", get_bloginfo('url'), $message);
                                        $message = str_replace("[[Date]]",gmdate("[Y-m-d] - H:i:s : ", time()) , $message);
                                        lws_sms_APISendSMS($model['sender'], $num, $message);
                                        break;
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }
}

//END SMS SCENARIOS//

//Add a new menu before Logout in the shop to put LWS SMS config
add_filter ( 'woocommerce_account_menu_items', 'lws_sms_config_woocommerce_new_menu', 40 );
function lws_sms_config_woocommerce_new_menu( $menu_links ){
	$menu_links = array_slice( $menu_links, 0, 5, true ) 
	+ array( 'lws-sms' => __('Alert SMS Settings', 'lws-sms') )
	+ array_slice( $menu_links, 5, NULL, true );
	
	return $menu_links;

}

//Initiate the CSS for our pages
function lws_sms_css_woocommerce_front() {
    wp_enqueue_style( 'wordpress_lws_css', plugins_url('/css/woocommerce.css', __FILE__));
}
add_action( 'wp_enqueue_scripts', 'lws_sms_css_woocommerce_front' );

// register permalink endpoint
add_action( 'init', 'lws_add_endpoint_sms' );
function lws_add_endpoint_sms() {
    //Add the endpoint, AKA the link to our page
	add_rewrite_endpoint( 'lws-sms', EP_PAGES );

}
// content for the new page in My Account, woocommerce_account_{ENDPOINT NAME}_endpoint
add_action( 'woocommerce_account_lws-sms_endpoint', 'lws_sms_lws_account_endpoint_content' );
function lws_sms_lws_account_endpoint_content() {
    lws_sms_checklist_model_client();
}

//Add the paramters to send SMS for the client
//add_action('woocommerce_account_dashboard', 'lws_sms_checklist_model_client');
function lws_sms_checklist_model_client(){
    
    //List of every variables used
    $user_id = get_current_user_id();
    $checked_options_client = array();
    $checked_options = array();
    $is_sms_active = false;

    //Getting all checkboxes for the client
    //If null, create it
    if (!get_user_meta( $user_id, 'checkboxes', true)){
        foreach (HOOKS as $hook){ 
            $checked_options_client[$hook]['is_checked'] = true; 
            }
        update_user_meta( $user_id, 'checkboxes', $checked_options_client);
    }
    $checked_options_client = get_user_meta( $user_id, 'checkboxes', true);
    
    //Getting all checkboxes status, creating it if needed
    if (!$checked_options = get_option('lws_checked_options')){
        foreach (HOOKS as $hook){
            $checked_options[$hook]['is_checked'] = false;
            $checked_options[$hook]['used'] = 'NO';
        }
        foreach (ADMIN_HOOKS as $hook){
            $checked_options[$hook]['is_checked'] = false;
            $checked_options[$hook]['used'] = 'NO';
        }
        update_option('lws_checked_options', $checked_options);
    }
    
    //Check if at least one option is active
    foreach (HOOKS as $hook){
        if ($checked_options[$hook]['is_checked']){
            $is_sms_active = true;
            break;
        }
    }
    
    //When submitted, check if checkboxes are checked
    //Update the options
    if (isset($_POST['update_sms_client'])){
        foreach (HOOKS as $hook){
            isset($_POST['client_order_' . $hook]) ? 
            $checked_options_client[$hook]['is_checked'] = true : $checked_options_client[$hook]['is_checked'] = false;
        }
        update_user_meta( $user_id, 'checkboxes', $checked_options_client);
    }

    include __DIR__ . '/view/account_checkbox_client.php';
}

    //WOOCOMMERCE//
